@@ -4,7 +4,6 @@ import com.unciv.Constants
 import com.unciv.UncivGame
 import com.unciv.UncivGame.Version
 import com.unciv.logic.BackwardCompatibility.convertFortify
-import com.unciv.logic.BackwardCompatibility.convertOldGameSpeed
 import com.unciv.logic.BackwardCompatibility.guaranteeUnitPromotions
 import com.unciv.logic.BackwardCompatibility.migrateToTileHistory
 import com.unciv.logic.BackwardCompatibility.removeMissingModReferences
@@ -65,8 +64,9 @@ data class CompatibilityVersion(
 
 }
 
-data class VictoryData(val winningCiv:String, val victoryType:String, val victoryTurn:Int){
-    constructor(): this("","",0) // for serializer
+data class VictoryData(val winningCiv: String, val victoryType: String, val victoryTurn: Int) {
+    @Suppress("unused")  // used by json serialization
+    constructor(): this("","",0)
 }
 
 class GameInfo : IsPartOfGameInfoSerialization, HasGameInfoSerializationVersion {
@@ -199,9 +199,11 @@ class GameInfo : IsPartOfGameInfoSerialization, HasGameInfoSerializationVersion
         }
     }
 
+    @delegate:Transient
+    val civMap by lazy { civilizations.associateBy { it.civName } }
     /** Get a civ by name
      *  @throws NoSuchElementException if no civ of that name is in the game (alive or dead)! */
-    fun getCivilization(civName: String) = civilizations.first { it.civName == civName }
+    fun getCivilization(civName: String) = civMap.getValue(civName)
     fun getCurrentPlayerCivilization() = currentPlayerCiv
     fun getCivilizationsAsPreviews() = civilizations.map { it.asPreview() }.toMutableList()
     /** Get barbarian civ
@@ -419,7 +421,7 @@ class GameInfo : IsPartOfGameInfoSerialization, HasGameInfoSerializationVersion
     }
 
     /** Generate a notification pointing out resources.
-     *  Used by [addTechnology][TechManager.addTechnology] and [ResourcesOverviewTab][com.unciv.ui.overviewscreen.ResourcesOverviewTab]
+     *  Used by [addTechnology][TechManager.addTechnology] and [ResourcesOverviewTab][com.unciv.ui.screens.overviewscreen.ResourcesOverviewTab]
      * @param maxDistance from next City, 0 removes distance limitation.
      * @param showForeign Disables filter to exclude foreign territory.
      * @return `false` if no resources were found and no notification was added.
@@ -508,8 +510,6 @@ class GameInfo : IsPartOfGameInfoSerialization, HasGameInfoSerializationVersion
 
         removeMissingModReferences()
 
-        convertOldGameSpeed()
-
         for (baseUnit in ruleset.units.values)
             baseUnit.ruleset = ruleset
 
@@ -546,16 +546,17 @@ class GameInfo : IsPartOfGameInfoSerialization, HasGameInfoSerializationVersion
 
         convertFortify()
 
-        for (civInfo in sequence {
+        for (civInfo in civilizations.asSequence()
             // update city-state resource first since the happiness of major civ depends on it.
             // See issue: https://github.com/yairm210/Unciv/issues/7781
-            yieldAll(civilizations.filter { it.isCityState() })
-            yieldAll(civilizations.filter { !it.isCityState() })
-        }) {
+            .sortedByDescending { it.isCityState() }
+        ) {
             for (unit in civInfo.units.getCivUnits())
                 unit.updateVisibleTiles(false) // this needs to be done after all the units are assigned to their civs and all other transients are set
-            if(civInfo.playerType == PlayerType.Human)
+            if (civInfo.playerType == PlayerType.Human)
                 civInfo.exploredRegion.setMapParameters(tileMap.mapParameters) // Required for the correct calculation of the explored region on world wrap maps
+
+            civInfo.cache.updateOurTiles()
             civInfo.cache.updateSightAndResources() // only run ONCE and not for each unit - this is a huge performance saver!
 
             // Since this depends on the cities of ALL civilizations,
@@ -586,10 +587,6 @@ class GameInfo : IsPartOfGameInfoSerialization, HasGameInfoSerializationVersion
 
                 cityInfo.cityStats.update()
             }
-
-            if (civInfo.hasEverOwnedOriginalCapital == null) {
-                civInfo.hasEverOwnedOriginalCapital = civInfo.cities.any { it.isOriginalCapital }
-            }
         }
 
         spaceResources.clear()
