@@ -3,7 +3,6 @@ package com.unciv.logic.automation.unit
 import com.badlogic.gdx.math.Vector2
 import com.unciv.Constants
 import com.unciv.UncivGame
-import com.unciv.logic.map.HexMath
 import com.unciv.logic.automation.Automation
 import com.unciv.logic.automation.ThreatLevel
 import com.unciv.logic.automation.civilization.NextTurnAutomation
@@ -12,13 +11,15 @@ import com.unciv.logic.city.City
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.map.BFS
+import com.unciv.logic.map.HexMath
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.tile.Terrain
 import com.unciv.models.ruleset.tile.TileImprovement
+import com.unciv.models.ruleset.unique.LocalUniqueCache
 import com.unciv.models.ruleset.unique.UniqueType
-import com.unciv.ui.worldscreen.unit.actions.UnitActions
+import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions
 import com.unciv.utils.Log
 import com.unciv.utils.debug
 
@@ -85,7 +86,7 @@ class WorkerAutomation(
             if (result.isEmpty())
                 debug("\tempty")
             else result.forEach {
-                debug("\t$it")    //  ${it.getCity()?.name} included in TileInfo toString()
+                debug("\t$it")    //  ${it.getCity()?.name} included in Tile toString()
             }
         }
         result
@@ -97,7 +98,7 @@ class WorkerAutomation(
      *
      *  value: The [BFS] searching from that city, whether successful or not.
      */
-    //todo: If BFS were to deal in vectors instead of TileInfos, we could copy this on cloning
+    //todo: If BFS were to deal in vectors instead of Tiles, we could copy this on cloning
     private val bfsCache = HashMap<Vector2, BFS>()
 
     //todo: UnitMovementAlgorithms.canReach still very expensive and could benefit from caching, it's not using BFS
@@ -235,7 +236,7 @@ class WorkerAutomation(
                 }
 
             while (true) {
-                for (cityTile in cityTilesToSeek.toList()) { // copy since we cahnge while running
+                for (cityTile in cityTilesToSeek.toList()) { // copy since we change while running
                     if (!bfs.hasReachedTile(cityTile)) continue
                     // we have a winner!
                     val pathToCity = bfs.getPathTo(cityTile)
@@ -370,51 +371,55 @@ class WorkerAutomation(
         }
         if (potentialTileImprovements.isEmpty()) return null
 
-        val uniqueImprovement = potentialTileImprovements.values.asSequence()
-            .filter { it.uniqueTo == civInfo.civName }
-            .maxByOrNull { Automation.rankStatsValue(it, unit.civ) }
+        val cityUniqueCaches = HashMap<City, LocalUniqueCache>()
+        fun getRankingWithImprovement(improvementName: String): Float {
+            val improvement = ruleSet.tileImprovements[improvementName]!!
+            val city = tile.getCity()
+            val cache =
+                    if (city == null) LocalUniqueCache(false)
+                    else cityUniqueCaches.getOrPut(city) { LocalUniqueCache() }
+            val stats = tile.stats.getImprovementStats(improvement, civInfo, tile.getCity(), cache)
+            return Automation.rankStatsValue(stats, unit.civ)
+        }
 
         val bestBuildableImprovement = potentialTileImprovements.values.asSequence()
-            .map { Pair(it, Automation.rankStatsValue(it, civInfo)) }
+            .map { Pair(it, getRankingWithImprovement(it.name)) }
             .filter { it.second > 0f }
             .maxByOrNull { it.second }?.first
 
-        val lastTerrain = tile.getLastTerrain()
+        val lastTerrain = tile.lastTerrain
 
         fun isUnbuildableAndRemovable(terrain: Terrain): Boolean = terrain.unbuildable
                 && ruleSet.tileImprovements.containsKey(Constants.remove + terrain.name)
 
+
         val improvementStringForResource: String? = when {
             tile.resource == null || !tile.hasViewableResource(civInfo) -> null
             tile.terrainFeatures.isNotEmpty()
                     && isUnbuildableAndRemovable(lastTerrain)
+                    && !tile.providesResources(civInfo)
                     && !isResourceImprovementAllowedOnFeature(tile, potentialTileImprovements) -> Constants.remove + lastTerrain.name
             else -> tile.tileResource.getImprovements().filter { it in potentialTileImprovements || it==tile.improvement }
-                .maxByOrNull { Automation.rankStatsValue(ruleSet.tileImprovements[it]!!, unit.civ) }
+                .maxByOrNull { getRankingWithImprovement(it) }
         }
 
+        // After gathering all the data, we conduct the hierarchy in one place
         val improvementString = when {
             tile.improvementInProgress != null -> tile.improvementInProgress!!
             improvementStringForResource != null -> if (improvementStringForResource==tile.improvement) null else improvementStringForResource
             // if this is a resource that HAS an improvement, but this unit can't build it, don't waste your time
             tile.resource != null && tile.tileResource.getImprovements().any() -> return null
-            tile.containsGreatImprovement() -> return null
-            tile.containsUnfinishedGreatImprovement() -> return null
+            bestBuildableImprovement == null -> null
 
-            // Defence is more important that civilian improvements
-            // While AI sucks in strategical placement of forts, allow a human does it manually
-            !civInfo.isHuman() && evaluateFortPlacement(tile, civInfo,false) -> Constants.fort
-            // I think we can assume that the unique improvement is better
-            uniqueImprovement != null && tile.improvementFunctions.canBuildImprovement(uniqueImprovement, civInfo)
-                -> uniqueImprovement.name
+            tile.improvement!=null && getRankingWithImprovement(tile.improvement!!) > getRankingWithImprovement(bestBuildableImprovement.name)
+                -> null // What we have is better, even if it's pillaged we should repair it
 
             lastTerrain.let {
                 isUnbuildableAndRemovable(it) &&
                         (Automation.rankStatsValue(it, civInfo) < 0 || it.hasUnique(UniqueType.NullifyYields) )
             } -> Constants.remove + lastTerrain.name
 
-            bestBuildableImprovement != null -> bestBuildableImprovement.name
-            else -> return null
+            else -> bestBuildableImprovement.name
         }
         return ruleSet.tileImprovements[improvementString] // For mods, the tile improvement may not exist, so don't assume.
     }
@@ -448,7 +453,7 @@ class WorkerAutomation(
             || tile.improvement == Constants.fort // don't build fort if it is already here
             || tile.hasViewableResource(civInfo) // don't build on resource tiles
             || tile.containsGreatImprovement() // don't build on great improvements (including citadel)
-            || tile.containsUnfinishedGreatImprovement()) return false
+        ) return false
 
         return true
     }
@@ -462,9 +467,9 @@ class WorkerAutomation(
         //todo Is the Citadel code dead anyway? If not - why does the nearestTiles check not respect the param?
 
         // build on our land only
-        if ((tile.owningCity?.civ != civInfo &&
+        if (tile.owningCity?.civ != civInfo &&
                     // except citadel which can be built near-by
-                    (!isCitadel || tile.neighbors.all { it.getOwner() != civInfo })) ||
+                    (!isCitadel || tile.neighbors.all { it.getOwner() != civInfo }) ||
             !isAcceptableTileForFort(tile)) return false
 
         // if this place is not perfect, let's see if there is a better one
@@ -484,7 +489,7 @@ class WorkerAutomation(
         val enemyCivs = civInfo.getKnownCivs()
             .filterNot { it == civInfo || it.cities.isEmpty() || !civInfo.getDiplomacyManager(it).canAttack() }
         // no potential enemies
-        if (enemyCivs.isEmpty()) return false
+        if (enemyCivs.none()) return false
 
         val threatMapping: (Civilization) -> Int = {
             // the war is already a good nudge to build forts
@@ -502,7 +507,7 @@ class WorkerAutomation(
             civInfo,
             it) <= threatMapping(it) }
         // no threat, let's not build fort
-        if (enemyCivsIsCloseEnough.isEmpty()) return false
+        if (enemyCivsIsCloseEnough.none()) return false
 
         // make list of enemy cities as sources of threat
         val enemyCities = mutableListOf<Tile>()
