@@ -20,7 +20,7 @@ import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
 import com.unciv.models.translations.equalsPlaceholderText
 import com.unciv.models.translations.getPlaceholderParameters
-import com.unciv.ui.utils.extensions.randomWeighted
+import com.unciv.ui.components.extensions.randomWeighted
 import kotlin.math.abs
 import kotlin.math.max
 import kotlin.math.min
@@ -173,13 +173,11 @@ class MapRegions (val ruleset: Ruleset){
             val nextRect = if (widerThanTall)
                 splitOffRegion.tileMap.getTilesInRectangle(Rectangle(
                         splitOffRegion.rect.x + splitPoint - 1, splitOffRegion.rect.y,
-                        1f, splitOffRegion.rect.height),
-                        rowsAndColumns = true)
+                        1f, splitOffRegion.rect.height))
             else
                 splitOffRegion.tileMap.getTilesInRectangle(Rectangle(
                         splitOffRegion.rect.x, splitOffRegion.rect.y + splitPoint - 1,
-                        splitOffRegion.rect.width, 1f),
-                        rowsAndColumns = true)
+                        splitOffRegion.rect.width, 1f))
 
             cumulativeFertility += if (splitOffRegion.continentID == -1)
                 nextRect.sumOf { it.getTileFertility(false) }
@@ -408,7 +406,7 @@ class MapRegions (val ruleset: Ruleset){
         val fallbackTiles = HashSet<Vector2>()
 
         // First check center
-        val centerTiles = region.tileMap.getTilesInRectangle(centerRect, rowsAndColumns = true)
+        val centerTiles = region.tileMap.getTilesInRectangle(centerRect)
         for (tile in centerTiles) {
             if (tileData[tile.position]!!.isTwoFromCoast)
                 continue // Don't even consider tiles two from coast
@@ -436,7 +434,7 @@ class MapRegions (val ruleset: Ruleset){
         }
 
         // Now check middle donut
-        val middleDonut = region.tileMap.getTilesInRectangle(middleRect, rowsAndColumns = true).filterNot { it in centerTiles }
+        val middleDonut = region.tileMap.getTilesInRectangle(middleRect).filterNot { it in centerTiles }
         riverTiles.clear()
         wetTiles.clear()
         dryTiles.clear()
@@ -467,7 +465,7 @@ class MapRegions (val ruleset: Ruleset){
         }
 
         // Now check the outer tiles. For these we don't care about rivers, coasts etc
-        val outerDonut = region.tileMap.getTilesInRectangle(region.rect, rowsAndColumns = true).filterNot { it in centerTiles || it in middleDonut}
+        val outerDonut = region.tileMap.getTilesInRectangle(region.rect).filterNot { it in centerTiles || it in middleDonut}
         dryTiles.clear()
         for (tile in outerDonut) {
             if (region.continentID != -1 && region.continentID != tile.getContinent())
@@ -648,9 +646,9 @@ class MapRegions (val ruleset: Ruleset){
             val validBonuses = ruleset.tileResources.values.filter {
                 it.resourceType == ResourceType.Bonus &&
                 it.food >= 1 &&
-                plot.getLastTerrain().name in it.terrainsCanBeFoundOn
+                plot.lastTerrain.name in it.terrainsCanBeFoundOn
             }
-            val goodPlotForOasis = canPlaceOasis && plot.getLastTerrain().name in oasisEquivalent!!.occursOn
+            val goodPlotForOasis = canPlaceOasis && plot.lastTerrain.name in oasisEquivalent!!.occursOn
 
             if (validBonuses.isNotEmpty() || goodPlotForOasis) {
                 if (goodPlotForOasis) {
@@ -701,7 +699,7 @@ class MapRegions (val ruleset: Ruleset){
 
                 if (plot.resource != null) continue
 
-                val bonusToPlace = stoneTypeBonuses.filter { plot.getLastTerrain().name in it.terrainsCanBeFoundOn }.randomOrNull()
+                val bonusToPlace = stoneTypeBonuses.filter { plot.lastTerrain.name in it.terrainsCanBeFoundOn }.randomOrNull()
                 if (bonusToPlace != null) {
                     plot.resource = bonusToPlace.name
                     stoneNeeded--
@@ -717,7 +715,7 @@ class MapRegions (val ruleset: Ruleset){
         val bestImprovementYield = tile.tileMap.ruleset!!.tileImprovements.values
                 .filter { !it.hasUnique(UniqueType.GreatImprovement) &&
                         it.uniqueTo == null &&
-                        tile.getLastTerrain().name in it.terrainsCanBeBuiltOn }
+                        tile.lastTerrain.name in it.terrainsCanBeBuiltOn }
                 .maxOfOrNull { it[stat] }
         return baseYield + (bestImprovementYield ?: 0f)
     }
@@ -1126,7 +1124,7 @@ class MapRegions (val ruleset: Ruleset){
         // Second place one (1) luxury at minor civ start locations
         // Check only ones that got a start location
         for (startLocation in tileMap.startingLocationsByNation
-                .filterKeys { ruleset.nations[it]!!.isCityState() }.map { it.value.first() }) {
+                .filterKeys { ruleset.nations[it]!!.isCityState }.map { it.value.first() }) {
             val region = regions.firstOrNull { startLocation in it.tiles }
             val tilesToCheck = startLocation.getTilesInDistanceRange(1..2)
             // 75% probability that we first attempt to place a "city state" luxury, then a random or regional one
@@ -1153,13 +1151,11 @@ class MapRegions (val ruleset: Ruleset){
         regionTargetNumber = max(1, regionTargetNumber)
         for (region in regions) {
             val resource = ruleset.tileResources[region.luxury] ?: continue
-            if (isWaterOnlyResource(resource))
-                tryAddingResourceToTiles(resource, regionTargetNumber,
-                        tileMap.getTilesInRectangle(region.rect).filter { it.isWater && it.neighbors.any { neighbor -> neighbor.getContinent() == region.continentID } }.shuffled(),
-                        0.4f, true, 4, 2)
-            else
-                tryAddingResourceToTiles(resource, regionTargetNumber, region.tiles.asSequence().shuffled(), 0.4f,
-                    true, 4, 2)
+            fun Tile.isShoreOfContinent(continent: Int) = isWater && neighbors.any { it.getContinent() == continent }
+            val candidates = if (isWaterOnlyResource(resource))
+                tileMap.getTilesInRectangle(region.rect).filter { it.isShoreOfContinent(region.continentID) }
+            else region.tiles.asSequence()
+            tryAddingResourceToTiles(resource, regionTargetNumber, candidates.shuffled(), 0.4f, true, 4, 2)
         }
         // Fourth add random luxuries
         if (randomLuxuries.isNotEmpty()) {
@@ -1319,7 +1315,7 @@ class MapRegions (val ruleset: Ruleset){
 
         if (modernOptions.any())
             for (cityStateLocation in tileMap.startingLocationsByNation
-                    .filterKeys { ruleset.nations[it]!!.isCityState() }.values.map { it.first() }) {
+                    .filterKeys { ruleset.nations[it]!!.isCityState }.values.map { it.first() }) {
                 val resourceToPlace = modernOptions.random()
                 totalPlaced[resourceToPlace] =
                         totalPlaced[resourceToPlace]!! + tryAddingResourceToTiles(resourceToPlace, 1, cityStateLocation.getTilesInDistanceRange(1..3))
@@ -1338,7 +1334,7 @@ class MapRegions (val ruleset: Ruleset){
                 continue
             val weightings = strategicResources.map {
                 if (fallbackStrategic) {
-                    if (tile.getLastTerrain().name in it.terrainsCanBeFoundOn) 1f else 0f
+                    if (tile.lastTerrain.name in it.terrainsCanBeFoundOn) 1f else 0f
                 } else {
                     val uniques = it.getMatchingUniques(UniqueType.MinorDepositWeighting, conditionalTerrain).toList()
                     uniques.sumOf { unique -> unique.params[0].toInt() }.toFloat()
@@ -1395,7 +1391,7 @@ class MapRegions (val ruleset: Ruleset){
             if(fallbackBonuses && resource.resourceType == ResourceType.Bonus) {
                 // Since we haven't been able to generate any rule-based lists, just generate new ones on the fly
                 // Increase impact to avoid clustering since there is no terrain type stratification.
-                val fallbackList = tileMap.values.filter { it.getLastTerrain().name in resource.terrainsCanBeFoundOn }.shuffled()
+                val fallbackList = tileMap.values.filter { it.lastTerrain.name in resource.terrainsCanBeFoundOn }.shuffled()
                 placeResourcesInTiles((20 * bonusMultiplier).toInt(), fallbackList, listOf(resource), 2 + extraImpact, 2 + extraImpact, false)
             }
         }
@@ -1444,7 +1440,7 @@ class MapRegions (val ruleset: Ruleset){
         for (tile in tiles) {
             val conditionalTerrain = StateForConditionals(attackedTile = tile)
             if (tile.resource == null &&
-                    tile.getLastTerrain().name in resource.terrainsCanBeFoundOn &&
+                    tile.lastTerrain.name in resource.terrainsCanBeFoundOn &&
                     !tile.getBaseTerrain().hasUnique(UniqueType.BlocksResources, conditionalTerrain) &&
                     !resource.hasUnique(UniqueType.NoNaturalGeneration, conditionalTerrain) &&
                     resource.getMatchingUniques(UniqueType.TileGenerationConditions).none {
@@ -1519,7 +1515,7 @@ class MapRegions (val ruleset: Ruleset){
         for (tile in tileList) {
             if (tile.resource != null ||
                     (testTerrains &&
-                            (tile.getLastTerrain().name !in resourceOptions.first().terrainsCanBeFoundOn ||
+                            (tile.lastTerrain.name !in resourceOptions.first().terrainsCanBeFoundOn ||
                             resourceOptions.first().hasUnique(UniqueType.NoNaturalGeneration, conditionalTerrain)) ) ||
                     tile.getBaseTerrain().hasUnique(UniqueType.BlocksResources, conditionalTerrain))
                 continue // Can't place here, can't be a fallback tile
@@ -1693,13 +1689,12 @@ class Region (val tileMap: TileMap, val rect: Rectangle, val continentID: Int =
         val columnHasTile = HashSet<Int>()
 
         tiles.clear()
-        for (tile in tileMap.getTilesInRectangle(rect, rowsAndColumns = true).filter {
-            continentID == -1 || it.getContinent() == continentID } ) {
+        for (tile in tileMap.getTilesInRectangle(rect).filter {
+                continentID == -1 || it.getContinent() == continentID } ) {
             val fertility = tile.getTileFertility(continentID != -1)
             tiles.add(tile)
             totalFertility += fertility
 
-
             if (affectedByWorldWrap)
                 columnHasTile.add(tile.getColumn())
 
