@@ -189,7 +189,7 @@ class MapUnit : IsPartOfGameInfoSerialization {
     }
 
     val type: UnitType
-        get() = baseUnit.getType()
+        get() = baseUnit.type
 
     fun baseUnit(): BaseUnit = baseUnit
     fun getMovementString(): String =
@@ -307,7 +307,10 @@ class MapUnit : IsPartOfGameInfoSerialization {
         }
 
         // Set equality automatically determines if anything changed - https://kotlinlang.org/api/latest/jvm/stdlib/kotlin.collections/-abstract-set/equals.html
-        if (updateCivViewableTiles && oldViewableTiles != viewableTiles)
+        if (updateCivViewableTiles && oldViewableTiles != viewableTiles
+                // Don't bother updating if all previous and current viewable tiles are within our borders
+                && (oldViewableTiles.any { it !in civ.cache.ourTilesAndNeighboringTiles }
+                        || viewableTiles.any { it !in civ.cache.ourTilesAndNeighboringTiles }))
             civ.cache.updateViewableTiles(explorerPosition)
     }
 
@@ -414,6 +417,14 @@ class MapUnit : IsPartOfGameInfoSerialization {
         return getMatchingUniques(UniqueType.HealAdjacentUnits).sumOf { it.params[0].toInt() }
     }
 
+    fun getHealAmountForCurrentTile() = when {
+        isEmbarked() -> 0 // embarked units can't heal
+        health >= 100 -> 0 // No need to heal if at max health
+        hasUnique(UniqueType.HealOnlyByPillaging, checkCivInfoUniques = true) -> 0
+        else -> rankTileForHealing(getTile())
+    }
+    fun canHealInCurrentTile() = getHealAmountForCurrentTile() > 0
+
     // Only military land units can truly "garrison"
     fun canGarrison() = isMilitary() && baseUnit.isLandUnit()
 
