@@ -68,7 +68,7 @@ class RulesetValidator(val ruleset: Ruleset) {
         }
 
         for (nation in ruleset.nations.values) {
-            if (nation.cities.isEmpty() && !nation.isSpectator() && !nation.isBarbarian()) {
+            if (nation.cities.isEmpty() && !nation.isSpectator && !nation.isBarbarian) {
                 lines += "${nation.name} can settle cities, but has no city names!"
             }
 
@@ -413,11 +413,6 @@ class RulesetValidator(val ruleset: Ruleset) {
             }
         }
 
-        @Suppress("DEPRECATION")
-        if (ruleset.modOptions.maxXPfromBarbarians != 30) {
-            lines.add("maxXPfromBarbarians is moved to the constants object, instead use: \nconstants: {\n    maxXPfromBarbarians: ${ruleset.modOptions.maxXPfromBarbarians},\n}", RulesetErrorSeverity.Warning)
-        }
-
         return lines
     }
 
@@ -577,7 +572,9 @@ class RulesetErrorList : ArrayList<RulesetError>() {
     fun isWarnUser() = getFinalSeverity() >= RulesetErrorSeverity.Warning
 
     fun getErrorText(unfiltered: Boolean = false) =
-            filter { unfiltered || it.errorSeverityToReport != RulesetErrorSeverity.WarningOptionsOnly }
+            getErrorText { unfiltered || it.errorSeverityToReport != RulesetErrorSeverity.WarningOptionsOnly }
+    fun getErrorText(filter: (RulesetError)->Boolean) =
+            filter(filter)
                 .sortedByDescending { it.errorSeverityToReport }
                 .joinToString("\n") { it.errorSeverityToReport.name + ": " + it.text }
 }
