@@ -9,7 +9,7 @@ import com.unciv.models.ruleset.tile.ResourceType
 import com.unciv.models.ruleset.unique.UniqueParameterType.Companion.guessTypeForTranslationWriter
 import com.unciv.models.stats.Stat
 import com.unciv.models.translations.TranslationFileWriter
-import com.unciv.ui.utils.extensions.filterCompositeLogic
+import com.unciv.ui.components.extensions.filterCompositeLogic
 
 // 'region' names beginning with an underscore are used here for a prettier "Structure window" - they go in front ot the rest.
 
@@ -75,7 +75,7 @@ enum class UniqueParameterType(
         private val knownValues = setOf("Wounded", Constants.barbarians, "City-State", "Embarked", "Non-City")
         override fun getErrorSeverity(parameterText: String, ruleset: Ruleset):
                 UniqueType.UniqueComplianceErrorSeverity? {
-            if ('{' in parameterText) // "{filter} {filter}" for and logic
+            if (parameterText.startsWith('{')) // "{filter} {filter}" for and logic
                 return parameterText.filterCompositeLogic({ getErrorSeverity(it, ruleset) }) { a, b -> maxOf(a, b) }
             if (parameterText in knownValues) return null
             if (ruleset.unitPromotions.values.any { it.hasUnique(parameterText) })
@@ -89,7 +89,7 @@ enum class UniqueParameterType(
     BaseUnitFilter("baseUnitFilter", "Melee") {
         override fun getErrorSeverity(parameterText: String, ruleset: Ruleset):
                 UniqueType.UniqueComplianceErrorSeverity? {
-            if ('{' in parameterText) // "{filter} {filter}" for and logic
+            if (parameterText.startsWith('{')) // "{filter} {filter}" for and logic
                 return parameterText.filterCompositeLogic({ getErrorSeverity(it, ruleset) }) { a, b -> maxOf(a, b) }
             if (UnitName.getErrorSeverity(parameterText, ruleset) == null) return null
             if (ruleset.units.values.any { it.uniques.contains(parameterText) }) return null
@@ -235,19 +235,6 @@ enum class UniqueParameterType(
             parameterText != "All" && getErrorSeverity(parameterText, ruleset) == null
     },
 
-        /** [UniqueType.PercentProductionConstructions], [UniqueType.PercentProductionConstructionsCities] */
-        @Deprecated("as of 3.17.10 - removed 3.18.5")
-        ConstructionFilter("constructionFilter", "Spaceship Part") {
-            override fun getErrorSeverity(
-                parameterText: String,
-                ruleset: Ruleset
-            ): UniqueType.UniqueComplianceErrorSeverity? {
-                if (BuildingFilter.getErrorSeverity(parameterText, ruleset) == null) return null
-                if (BaseUnitFilter.getErrorSeverity(parameterText, ruleset) == null) return null
-                return UniqueType.UniqueComplianceErrorSeverity.WarningOnly
-            }
-        },
-
     /** Implemented by [PopulationManager.getPopulationFilterAmount][com.unciv.logic.city.CityPopulationManager.getPopulationFilterAmount] */
     PopulationFilter("populationFilter", "Followers of this Religion", null, "Population Filters") {
         private val knownValues = setOf("Population", "Specialists", "Unemployed", "Followers of the Majority Religion", "Followers of this Religion")
@@ -261,7 +248,7 @@ enum class UniqueParameterType(
         override fun getTranslationWriterStringsForOutput() = knownValues
     },
 
-    /** Implemented by [TileInfo.matchesTerrainFilter][com.unciv.logic.map.TileInfo.matchesTerrainFilter] */
+    /** Implemented by [Tile.matchesTerrainFilter][com.unciv.logic.map.tile.Tile.matchesTerrainFilter] */
     TerrainFilter("terrainFilter", Constants.freshWaterFilter, null, "Terrain Filters") {
         private val knownValues = setOf("All",
                 Constants.coastal, "River", "Open terrain", "Rough terrain", "Water resource",
@@ -282,7 +269,7 @@ enum class UniqueParameterType(
         override fun getTranslationWriterStringsForOutput() = knownValues
     },
 
-    /** Implemented by [TileInfo.matchesFilter][com.unciv.logic.map.TileInfo.matchesFilter] */
+    /** Implemented by [Tile.matchesFilter][com.unciv.logic.map.tile.Tile.matchesFilter] */
     TileFilter("tileFilter", "Farm", "Anything that can be used either in an improvementFilter or in a terrainFilter can be used here, plus 'unimproved'", "Tile Filters") {
         private val knownValues = setOf("unimproved", "All Road", "Great Improvement")
         override fun getErrorSeverity(parameterText: String, ruleset: Ruleset):
@@ -452,7 +439,7 @@ enum class UniqueParameterType(
         }
     },
 
-    /** [UniqueType.ConditionalPolicy] and others, no central implementation */
+    /** [UniqueType.ConditionalAfterPolicy] and others, no central implementation */
     Policy("policy", "Oligarchy", "The name of any policy") {
         override fun getErrorSeverity(
             parameterText: String,
