@@ -12,10 +12,11 @@ import com.unciv.models.ruleset.RulesetObject
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
-import com.unciv.ui.civilopedia.FormattedLine
-import com.unciv.ui.utils.extensions.filterAndLogic
-import com.unciv.ui.utils.extensions.getNeedMoreAmountString
-import com.unciv.ui.utils.extensions.toPercent
+import com.unciv.ui.components.extensions.filterAndLogic
+import com.unciv.ui.components.extensions.getNeedMoreAmountString
+import com.unciv.ui.components.extensions.toPercent
+import com.unciv.ui.objectdescriptions.BaseUnitDescriptions
+import com.unciv.ui.screens.civilopediascreen.FormattedLine
 import kotlin.math.pow
 
 // This is BaseUnit because Unit is already a base Kotlin class and to avoid mixing the two up
@@ -33,7 +34,8 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
     var range: Int = 2
     var interceptRange = 0
     var unitType: String = ""
-    fun getType() = ruleset.unitTypes[unitType]!!
+
+    val type by lazy { ruleset.unitTypes[unitType]!! }
     override var requiredTech: String? = null
     private var requiredResource: String? = null
 
@@ -84,6 +86,8 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
 
     override fun canBePurchasedWithStat(city: City?, stat: Stat): Boolean {
         if (city == null) return super.canBePurchasedWithStat(city, stat)
+        if (getRejectionReasons(city.civ, city).any { it.type != RejectionReasonType.Unbuildable  })
+            return false
         if (costFunctions.canBePurchasedWithStat(city, stat)) return true
         return super.canBePurchasedWithStat(city, stat)
     }
@@ -137,30 +141,34 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
         yieldAll(getRejectionReasons(civInfo, cityConstructions.city))
     }
 
-    fun getRejectionReasons(civ: Civilization, city:City?=null): Sequence<RejectionReason> = sequence {
+    fun getRejectionReasons(civ: Civilization, city:City?=null): Sequence<RejectionReason> {
+        val result = mutableListOf<RejectionReason>()
         if (requiredTech != null && !civ.tech.isResearched(requiredTech!!))
-            yield(RejectionReasonType.RequiresTech.toInstance("$requiredTech not researched"))
+            result.add(RejectionReasonType.RequiresTech.toInstance("$requiredTech not researched"))
         if (obsoleteTech != null && civ.tech.isResearched(obsoleteTech!!))
-            yield(RejectionReasonType.Obsoleted.toInstance("Obsolete by $obsoleteTech"))
+            result.add(RejectionReasonType.Obsoleted.toInstance("Obsolete by $obsoleteTech"))
 
         if (uniqueTo != null && uniqueTo != civ.civName)
-            yield(RejectionReasonType.UniqueToOtherNation.toInstance("Unique to $uniqueTo"))
+            result.add(RejectionReasonType.UniqueToOtherNation.toInstance("Unique to $uniqueTo"))
         if (civ.cache.uniqueUnits.any { it.replaces == name })
-            yield(RejectionReasonType.ReplacedByOurUnique.toInstance("Our unique unit replaces this"))
+            result.add(RejectionReasonType.ReplacedByOurUnique.toInstance("Our unique unit replaces this"))
 
         if (!civ.gameInfo.gameParameters.nuclearWeaponsEnabled && isNuclearWeapon())
-            yield(RejectionReasonType.DisabledBySetting.toInstance())
+            result.add(RejectionReasonType.DisabledBySetting.toInstance())
 
         for (unique in uniqueObjects.filter { it.conditionalsApply(civ, city) }) {
             when (unique.type) {
                 UniqueType.Unbuildable ->
-                    yield(RejectionReasonType.Unbuildable.toInstance())
+                    result.add(RejectionReasonType.Unbuildable.toInstance())
 
                 UniqueType.FoundCity -> if (civ.isCityState() || civ.isOneCityChallenger())
-                    yield(RejectionReasonType.NoSettlerForOneCityPlayers.toInstance())
+                    result.add(RejectionReasonType.NoSettlerForOneCityPlayers.toInstance())
 
-                UniqueType.MaxNumberBuildable -> if (civ.civConstructions.countConstructedObjects(this@BaseUnit) >= unique.params[0].toInt())
-                    yield(RejectionReasonType.MaxNumberBuildable.toInstance())
+                UniqueType.MaxNumberBuildable -> if (civ.civConstructions.countConstructedObjects(
+                            this@BaseUnit
+                        ) >= unique.params[0].toInt()
+                )
+                    result.add(RejectionReasonType.MaxNumberBuildable.toInstance())
 
                 else -> {}
             }
@@ -170,18 +178,29 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
             for ((resource, requiredAmount) in getResourceRequirements()) {
                 val availableAmount = civ.getCivResourcesByName()[resource]!!
                 if (availableAmount < requiredAmount) {
-                    yield(RejectionReasonType.ConsumesResources.toInstance(resource.getNeedMoreAmountString(requiredAmount - availableAmount)))
+                    result.add(
+                        RejectionReasonType.ConsumesResources.toInstance(
+                            resource.getNeedMoreAmountString(
+                                requiredAmount - availableAmount
+                            )
+                        )
+                    )
                 }
             }
         }
 
         for (unique in civ.getMatchingUniques(UniqueType.CannotBuildUnits))
             if (this@BaseUnit.matchesFilter(unique.params[0])) {
-                if (unique.conditionals.any { it.type == UniqueType.ConditionalBelowHappiness }){
-                    yield(RejectionReasonType.CannotBeBuilt.toInstance(unique.text, true))
-                }
-                else yield(RejectionReasonType.CannotBeBuilt.toInstance())
+                if (unique.conditionals.any { it.type == UniqueType.ConditionalBelowHappiness }) {
+                    result.add(
+                        RejectionReasonType.CannotBeBuilt.toInstance(
+                            unique.text,
+                            true
+                        )
+                    )
+                } else result.add(RejectionReasonType.CannotBeBuilt.toInstance())
             }
+        return result.asSequence()
     }
 
     fun isBuildable(civInfo: Civilization) = getRejectionReasons(civInfo).none()
@@ -275,11 +294,10 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
             "non-air" -> !movesLikeAirUnits()
 
             "Nuclear Weapon" -> isNuclearWeapon()
-            // "Great" should be deprecated, replaced by "Great Person".
-            "Great Person", "Great" -> isGreatPerson()
+            "Great Person" -> isGreatPerson()
             "Religious" -> hasUnique(UniqueType.ReligiousUnit)
             else -> {
-                if (getType().matchesFilter(filter)) return true
+                if (type.matchesFilter(filter)) return true
                 if (
                     // Uniques using these kinds of filters should be deprecated and replaced with adjective-only parameters
                     filter.endsWith(" units")
@@ -296,7 +314,7 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
 
     fun isNuclearWeapon() = hasUnique(UniqueType.NuclearWeapon)
 
-    fun movesLikeAirUnits() = getType().getMovementType() == UnitMovementType.Air
+    fun movesLikeAirUnits() = type.getMovementType() == UnitMovementType.Air
 
     /** Returns resource requirements from both uniques and requiredResource field */
     override fun getResourceRequirements(): HashMap<String, Int> = resourceRequirementsInternal
@@ -316,15 +334,15 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
     fun isMilitary() = isRanged() || isMelee()
     fun isCivilian() = !isMilitary()
 
-    val isLandUnitInternal by lazy { getType().isLandUnit() }
+    val isLandUnitInternal by lazy { type.isLandUnit() }
     fun isLandUnit() = isLandUnitInternal
-    fun isWaterUnit() = getType().isWaterUnit()
-    fun isAirUnit() = getType().isAirUnit()
+    fun isWaterUnit() = type.isWaterUnit()
+    fun isAirUnit() = type.isAirUnit()
 
     fun isProbablySiegeUnit() =
         (
             isRanged()
-            && (uniqueObjects + getType().uniqueObjects)
+            && (uniqueObjects + type.uniqueObjects)
                 .any { it.isOfType(UniqueType.Strength)
                     && it.params[0].toInt() > 0
                     && it.conditionals.any { conditional -> conditional.isOfType(UniqueType.ConditionalVsCity) }
